package io.xjar;

import javax.crypto.CipherOutputStream;
import java.io.Closeable;
import javax.crypto.CipherInputStream;
import java.security.spec.AlgorithmParameterSpec;
import java.security.Key;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import javax.crypto.Cipher;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.File;
import io.xjar.key.XKey;

public class XJdkDecryptor implements XDecryptor
{
    @Override
    public void decrypt(final XKey key, final File src, final File dest) throws IOException {
        if (!dest.getParentFile().exists() && !dest.getParentFile().mkdirs()) {
            throw new IOException("could not make directory: " + dest.getParentFile());
        }
        try (InputStream in = new FileInputStream(src);
             OutputStream out = new FileOutputStream(dest)) {
            this.decrypt(key, in, out);
        }
    }
    
    @Override
    public void decrypt(final XKey key, final InputStream in, final OutputStream out) throws IOException {
        CipherInputStream cis = null;
        try {
            final String algorithm = key.getAlgorithm();
            final Cipher cipher = Cipher.getInstance(algorithm);
            cipher.init(2, new SecretKeySpec(key.getDecryptKey(), algorithm.split("[/]")[0]), new IvParameterSpec(key.getIvParameter()));
            cis = new CipherInputStream(in, cipher);
            XKit.transfer(cis, out);
        }
        catch (final Exception e) {
            throw new IOException(e);
        }
        finally {
            XKit.close(cis);
        }
    }
    
    @Override
    public InputStream decrypt(final XKey key, final InputStream in) throws IOException {
        try {
            final String algorithm = key.getAlgorithm();
            final Cipher cipher = Cipher.getInstance(algorithm);
            cipher.init(2, new SecretKeySpec(key.getDecryptKey(), algorithm.split("[/]")[0]), new IvParameterSpec(key.getIvParameter()));
            return new CipherInputStream(in, cipher);
        }
        catch (final Exception e) {
            throw new IOException(e);
        }
    }
    
    @Override
    public OutputStream decrypt(final XKey key, final OutputStream out) throws IOException {
        try {
            final String algorithm = key.getAlgorithm();
            final Cipher cipher = Cipher.getInstance(algorithm);
            cipher.init(2, new SecretKeySpec(key.getDecryptKey(), algorithm.split("[/]")[0]), new IvParameterSpec(key.getIvParameter()));
            return new CipherOutputStream(out, cipher);
        }
        catch (final Exception e) {
            throw new IOException(e);
        }
    }
}
